﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.SQLite;
using System.IO;
using System.Data;
using System.Diagnostics;

namespace Test
{
  class Program
  {
    static void Main(string[] args)
    {
      var output_file = Path.GetTempFileName();

      System.Data.IDbConnection Connection;
      string connection_string = string.Format( "Data Source={0}; Version=3; Connect Timeout=5; Read Only=FALSE", output_file );

      Connection = new SQLiteConnection( connection_string );
      Connection.Open();

      //create table, add one row
      using( var cmd = Connection.CreateCommand() )
      {
        cmd.CommandText =
@"CREATE TABLE Test (
  ID INTEGER NOT NULL,
  C1 INTEGER,
  C2 INTEGER,
  C3 INTEGER,
  C3a INT,
  C3b UINT32,
  C4 INTEGER,
  PRIMARY KEY (ID ASC)
);";
        cmd.ExecuteNonQuery();
      }

      using( var cmd = Connection.CreateCommand() )
      {
        cmd.CommandText = "INSERT INTO Test Values ( 1,0,0,0,0,0,0 );";
        cmd.ExecuteNonQuery();
      }

      //run parameterized inserts with max 8,16,32,64-bit params
      using( var cmd = Connection.CreateCommand() )
      {
        cmd.CommandText = @" UPDATE Test SET C1 = @p1, C2=@p2, C3=@p3, C3a=@p3, C3b=@p3, C4=@p4 WHERE ID = 1";

        var param1 = cmd.CreateParameter();
        param1.ParameterName = "@p1";
        param1.DbType = DbType.Byte;
        param1.Value = byte.MaxValue;

        var param2 = cmd.CreateParameter();
        param2.ParameterName = "@p2";
        param2.DbType = DbType.UInt16;
        param2.Value = UInt16.MaxValue;

        var param3 = cmd.CreateParameter();
        param3.ParameterName = "@p3";
        param3.DbType = DbType.UInt32;
        param3.Value = UInt32.MaxValue;

        var param4 = cmd.CreateParameter();
        param4.ParameterName = "@p4";
        param4.DbType = DbType.UInt64;
        param4.Value = UInt64.MaxValue;

        cmd.Parameters.Add( param1 );
        cmd.Parameters.Add( param2 );
        cmd.Parameters.Add( param3 );

        string message = string.Format( 
          "Reading values:\nC1 - {0:x}\nC2 - {1:x}\nC3 - {2:x}\nC4 - {3:x}",
          param1.Value, param2.Value, param3.Value, param4.Value );
        System.Console.WriteLine( message );
        
        cmd.Parameters.Add( param4 );
        cmd.ExecuteNonQuery();
      }

      //check Convert.Blah
      using( var cmd = Connection.CreateCommand() )
      {
        cmd.CommandText = @" SELECT C1, C2, C3, C3a, C3b, C4 FROM Test WHERE ID=1";
        using( var reader = cmd.ExecuteReader() )
        {
          while( reader.Read() )
          {
            string message = string.Format( 
              "Reading values:\nC1 - {0:x}\nC2 - {1:x}\nC3 - {2:x}\nC3a - {3:x}\nC3b - {4:x}\nC4 - {5:x}",
              reader[0], reader[1], reader[2], reader[3], reader[4], reader[5] );
            System.Console.WriteLine( message );

            try{

              //C1 - pass
              Trace.Assert( (long)reader[0] == byte.MaxValue , "byte.MaxValue not saved" );
              Trace.Assert( Convert.ToByte( reader[0] ) == byte.MaxValue , "byte.MaxValue not saved" );

              //C2 - pass
              Trace.Assert( (long)reader[1] == UInt16.MaxValue, "UInt16.MaxValue not saved" );
              Trace.Assert( Convert.ToUInt16( reader[1] ) == UInt16.MaxValue, "UInt16.MaxValue not saved" );

              //C3 - fail
              Trace.Assert( (long)reader[2] == UInt32.MaxValue, "UInt32.MaxValue not saved" ); //sign extension
              Trace.Assert( Convert.ToUInt32( reader[2] ) == UInt32.MaxValue, "UInt32.MaxValue not saved" ); //convert throws an overflow exception

              //C3a - partial
              Trace.Assert( (UInt32)(int)reader[3] == UInt32.MaxValue, "UInt32.MaxValue not saved" ); //pass
              Trace.Assert( Convert.ToUInt32( reader[3] ) == UInt32.MaxValue, "UInt32.MaxValue not saved" ); //convert throws an out-of-range exception

              //C3b - Didn't test this agains the trunk.  I'll assume it works.
              Trace.Assert( (UInt32)reader[4] == UInt32.MaxValue, "UInt32.MaxValue not saved" );
              Trace.Assert( Convert.ToUInt32( reader[4] ) == UInt32.MaxValue, "UInt32.MaxValue not saved" );

              //C4 - fail
              Trace.Assert( (UInt32)(long)reader[5] == UInt32.MaxValue, "UInt32.MaxValue not saved" ); //pass
              Trace.Assert( Convert.ToUInt64( reader[5] ) == UInt32.MaxValue, "UInt32.MaxValue not saved" ); //fails, but I didn't really expect this one to work
            }
            catch( Exception e )
            {
              System.Console.Write( string.Format( "Conversion failed:\n{0}", e.Message ) );
            }
          }
        }
      }
    }
  }
}
